@@ -34,7 +34,6 @@ ALL_TABLES = [
 
 
 class MetricsBuilder:
-
     NUM_TPCH_QUESTIONS = 10
 
     HEADERS = [
@@ -114,7 +113,7 @@ class MetricsBuilder:
 
 def get_df_with_parquet_folder(parquet_folder: str) -> Callable[[str], DataFrame]:
     def _get_df(table_name: str) -> DataFrame:
-        return DataFrame.read_parquet(os.path.join(parquet_folder, table_name, "*.parquet"))
+        return daft.read_parquet(os.path.join(parquet_folder, table_name, "*.parquet"))
 
     return _get_df
 
@@ -178,6 +177,7 @@ def warmup_environment(requirements: str | None, parquet_folder: str):
         )
 
         logger.info("Warming up Ray cluster with a function...")
+
         # NOTE: installation of runtime_env is supposed to be eager but it seems to be happening async.
         # Here we farm out some work on Ray to warm up all the workers by downloading data.
         # Warm up n := num_cpus workers.
@@ -231,8 +231,13 @@ if __name__ == "__main__":
         default=None,
         help="Path to pip-style requirements.txt file to bootstrap environment on remote Ray cluster",
     )
-    args = parser.parse_args()
+    parser.add_argument(
+        "--skip_warmup",
+        action="store_true",
+        help="Skip warming up data before benchmark",
+    )
 
+    args = parser.parse_args()
     if args.output_csv_headers:
         warnings.warn("Detected --output_csv_headers flag, but this flag is deprecated - CSVs always output headers")
 
@@ -248,7 +253,10 @@ if __name__ == "__main__":
     else:
         parquet_folder = generate_parquet_data(args.tpch_gen_folder, args.scale_factor, num_parts)
 
-    warmup_environment(args.requirements, parquet_folder)
+    if args.skip_warmup:
+        warnings.warn("Detected --skip_warmup flag, skipping warm up task")
+    else:
+        warmup_environment(args.requirements, parquet_folder)
 
     run_all_benchmarks(
         parquet_folder,
