@@ -1,12 +1,16 @@
 from __future__ import annotations
 
 import builtins
+from typing import TYPE_CHECKING
 
 import pyarrow as pa
 
 from daft.context import get_context
 from daft.daft import ImageMode, PyDataType, PyTimeUnit
 
+if TYPE_CHECKING:
+    import numpy as np
+
 _RAY_DATA_EXTENSIONS_AVAILABLE = True
 _TENSOR_EXTENSION_TYPES = []
 try:
@@ -179,6 +183,11 @@ class DataType:
         """Creates the Null DataType: Always the ``Null`` value"""
         return cls._from_pydatatype(PyDataType.null())
 
+    @classmethod
+    def decimal128(cls, precision: int, scale: int) -> DataType:
+        """Fixed-precision decimal."""
+        return cls._from_pydatatype(PyDataType.decimal128(precision, scale))
+
     @classmethod
     def date(cls) -> DataType:
         """Create a Date DataType: A date with a year, month and day"""
@@ -195,16 +204,16 @@ class DataType:
         return cls._from_pydatatype(PyDataType.duration(timeunit._timeunit))
 
     @classmethod
-    def list(cls, name: str, dtype: DataType) -> DataType:
+    def list(cls, dtype: DataType) -> DataType:
         """Create a List DataType: Variable-length list, where each element in the list has type ``dtype``
 
         Args:
             dtype: DataType of each element in the list
         """
-        return cls._from_pydatatype(PyDataType.list(name, dtype._dtype))
+        return cls._from_pydatatype(PyDataType.list(dtype._dtype))
 
     @classmethod
-    def fixed_size_list(cls, name: str, dtype: DataType, size: int) -> DataType:
+    def fixed_size_list(cls, dtype: DataType, size: int) -> DataType:
         """Create a FixedSizeList DataType: Fixed-size list, where each element in the list has type ``dtype``
         and each list has length ``size``.
 
@@ -214,7 +223,7 @@ class DataType:
         """
         if not isinstance(size, int) or size <= 0:
             raise ValueError("The size for a fixed-size list must be a positive integer, but got: ", size)
-        return cls._from_pydatatype(PyDataType.fixed_size_list(name, dtype._dtype, size))
+        return cls._from_pydatatype(PyDataType.fixed_size_list(dtype._dtype, size))
 
     @classmethod
     def struct(cls, fields: dict[str, DataType]) -> DataType:
@@ -230,7 +239,7 @@ class DataType:
         return cls._from_pydatatype(PyDataType.extension(name, storage_dtype._dtype, metadata))
 
     @classmethod
-    def embedding(cls, name: str, dtype: DataType, size: int) -> DataType:
+    def embedding(cls, dtype: DataType, size: int) -> DataType:
         """Create an Embedding DataType: embeddings are fixed size arrays, where each element
         in the array has a **numeric** ``dtype`` and each array has a fixed length of ``size``.
 
@@ -240,7 +249,7 @@ class DataType:
         """
         if not isinstance(size, int) or size <= 0:
             raise ValueError("The size for a embedding must be a positive integer, but got: ", size)
-        return cls._from_pydatatype(PyDataType.embedding(name, dtype._dtype, size))
+        return cls._from_pydatatype(PyDataType.embedding(dtype._dtype, size))
 
     @classmethod
     def image(
@@ -283,6 +292,30 @@ class DataType:
             )
         return cls._from_pydatatype(PyDataType.image(mode, height, width))
 
+    @classmethod
+    def tensor(
+        cls,
+        dtype: DataType,
+        shape: tuple[int, ...] | None = None,
+    ) -> DataType:
+        """Create a tensor DataType: tensor arrays contain n-dimensional arrays of data of the provided ``dtype`` as elements, each of the provided
+        ``shape``.
+
+        If a ``shape`` is given, each ndarray in the column will have this shape.
+
+        If ``shape`` is not given, the ndarrays in the column can have different shapes. This is much more flexible,
+        but will result in a less compact representation and may be make some operations less efficient.
+
+        Args:
+            dtype: The type of the data contained within the tensor elements.
+            shape: The shape of each tensor in the column. This is ``None`` by default, which allows the shapes of
+                each tensor element to vary.
+        """
+        if shape is not None:
+            if not isinstance(shape, tuple) or not shape or any(not isinstance(n, int) for n in shape):
+                raise ValueError("Tensor shape must be a non-empty tuple of ints, but got: ", shape)
+        return cls._from_pydatatype(PyDataType.tensor(dtype._dtype, shape))
+
     @classmethod
     def from_arrow_type(cls, arrow_type: pa.lib.DataType) -> DataType:
         """Maps a PyArrow DataType to a Daft DataType"""
@@ -314,6 +347,8 @@ class DataType:
             return cls.bool()
         elif pa.types.is_null(arrow_type):
             return cls.null()
+        elif pa.types.is_decimal128(arrow_type):
+            return cls.decimal128(arrow_type.precision, arrow_type.scale)
         elif pa.types.is_date32(arrow_type):
             return cls.date()
         elif pa.types.is_timestamp(arrow_type):
@@ -325,18 +360,22 @@ class DataType:
         elif pa.types.is_list(arrow_type) or pa.types.is_large_list(arrow_type):
             assert isinstance(arrow_type, (pa.ListType, pa.LargeListType))
             field = arrow_type.value_field
-            return cls.list(field.name, cls.from_arrow_type(field.type))
+            return cls.list(cls.from_arrow_type(field.type))
         elif pa.types.is_fixed_size_list(arrow_type):
             assert isinstance(arrow_type, pa.FixedSizeListType)
             field = arrow_type.value_field
-            return cls.fixed_size_list(field.name, cls.from_arrow_type(field.type), arrow_type.list_size)
+            return cls.fixed_size_list(cls.from_arrow_type(field.type), arrow_type.list_size)
         elif pa.types.is_struct(arrow_type):
             assert isinstance(arrow_type, pa.StructType)
             fields = [arrow_type[i] for i in range(arrow_type.num_fields)]
             return cls.struct({field.name: cls.from_arrow_type(field.type) for field in fields})
         elif _RAY_DATA_EXTENSIONS_AVAILABLE and isinstance(arrow_type, tuple(_TENSOR_EXTENSION_TYPES)):
-            # TODO(Clark): Add a native cross-lang extension type representation for Ray's tensor extension types.
-            return cls.python()
+            scalar_dtype = cls.from_arrow_type(arrow_type.scalar_type)
+            shape = arrow_type.shape if isinstance(arrow_type, ArrowTensorType) else None
+            return cls.tensor(scalar_dtype, shape)
+        elif isinstance(arrow_type, getattr(pa, "FixedShapeTensorType", ())):
+            scalar_dtype = cls.from_arrow_type(arrow_type.value_type)
+            return cls.tensor(scalar_dtype, tuple(arrow_type.shape))
         elif isinstance(arrow_type, pa.PyExtensionType):
             # TODO(Clark): Add a native cross-lang extension type representation for PyExtensionTypes.
             raise ValueError(
@@ -359,22 +398,30 @@ class DataType:
                 metadata = arrow_type.__arrow_ext_serialize__().decode()
             except AttributeError:
                 metadata = None
-            return cls.extension(
-                name,
-                cls.from_arrow_type(arrow_type.storage_type),
-                metadata,
-            )
+
+            if name == "daft.super_extension":
+                assert metadata is not None
+                return cls._from_pydatatype(PyDataType.from_json(metadata))
+            else:
+                return cls.extension(
+                    name,
+                    cls.from_arrow_type(arrow_type.storage_type),
+                    metadata,
+                )
         else:
             # Fall back to a Python object type.
             # TODO(Clark): Add native support for remaining Arrow types.
             return cls.python()
 
     @classmethod
-    def from_numpy_dtype(cls, np_type) -> DataType:
+    def from_numpy_dtype(cls, np_type: np.dtype) -> DataType:
         """Maps a Numpy datatype to a Daft DataType"""
         arrow_type = pa.from_numpy_dtype(np_type)
         return cls.from_arrow_type(arrow_type)
 
+    def to_arrow_dtype(self, cast_tensor_to_ray_type: builtins.bool = False) -> pa.DataType:
+        return self._dtype.to_arrow(cast_tensor_to_ray_type)
+
     @classmethod
     def python(cls) -> DataType:
         """Create a Python DataType: a type which refers to an arbitrary Python object"""
@@ -387,6 +434,18 @@ class DataType:
 
         return self == DataType.python()
 
+    def _is_tensor_type(self) -> builtins.bool:
+        return self._dtype.is_tensor()
+
+    def _is_fixed_shape_tensor_type(self) -> builtins.bool:
+        return self._dtype.is_fixed_shape_tensor()
+
+    def _is_image_type(self) -> builtins.bool:
+        return self._dtype.is_image()
+
+    def _is_fixed_shape_image_type(self) -> builtins.bool:
+        return self._dtype.is_fixed_shape_image()
+
     def _is_logical_type(self) -> builtins.bool:
         return self._dtype.is_logical()
 
