@@ -9,7 +9,7 @@ import fsspec
 import pyarrow as pa
 
 from daft import context
-from daft.daft import ImageFormat
+from daft.daft import CountMode, ImageFormat
 from daft.daft import PyExpr as _PyExpr
 from daft.daft import col as _col
 from daft.daft import lit as _lit
@@ -275,8 +275,8 @@ class Expression:
         expr = self._expr.cast(dtype._dtype)
         return Expression._from_pyexpr(expr)
 
-    def _count(self) -> Expression:
-        expr = self._expr.count()
+    def _count(self, mode: CountMode = CountMode.Valid) -> Expression:
+        expr = self._expr.count(mode)
         return Expression._from_pyexpr(expr)
 
     def _sum(self) -> Expression:
@@ -379,6 +379,9 @@ class Expression:
     def _to_field(self, schema: Schema) -> Field:
         return Field._from_pyfield(self._expr.to_field(schema._schema))
 
+    def __hash__(self) -> int:
+        return self._expr.__hash__()
+
     def __getstate__(self) -> bytes:
         return self._expr.__getstate__()
 
@@ -614,6 +617,14 @@ class ExpressionListNamespace(ExpressionNamespace):
         delimiter_expr = Expression._to_expression(delimiter)
         return Expression._from_pyexpr(self._expr.list_join(delimiter_expr._expr))
 
+    def lengths(self) -> Expression:
+        """Gets the length of each list
+
+        Returns:
+            Expression: a UInt64 expression which is the length of each list
+        """
+        return Expression._from_pyexpr(self._expr.list_lengths())
+
 
 class ExpressionsProjection(Iterable[Expression]):
     """A collection of Expressions that can be projected onto a Table to produce another Table
@@ -716,6 +727,9 @@ class ExpressionsProjection(Iterable[Expression]):
             raise ValueError(f"{name} not found in ExpressionsProjection")
         return self._output_name_to_exprs[name]
 
+    def to_inner_py_exprs(self) -> list[_PyExpr]:
+        return [expr._expr for expr in self]
+
     def resolve_schema(self, schema: Schema) -> Schema:
         fields = [e._to_field(schema) for e in self]
         return Schema._from_field_name_and_types([(f.name, f.dtype) for f in fields])
@@ -768,3 +782,24 @@ class ExpressionImageNamespace(ExpressionNamespace):
         if not isinstance(h, int):
             raise TypeError(f"expected int for h but got {type(h)}")
         return Expression._from_pyexpr(self._expr.image_resize(w, h))
+
+    def crop(self, bbox: tuple[int, int, int, int] | Expression) -> Expression:
+        """
+        Crops images with the provided bounding box
+
+        Args:
+            bbox (tuple[float, float, float, float] | Expression): Either a tuple of (x, y, width, height)
+                parameters for cropping, or a List Expression where each element is a length 4 List
+                which represents the bounding box for the crop
+
+        Returns:
+            Expression: An Image expression representing the cropped image
+        """
+        if not isinstance(bbox, Expression):
+            if len(bbox) != 4 or not all([isinstance(x, int) for x in bbox]):
+                raise ValueError(
+                    f"Expected `bbox` to be either a tuple of 4 ints or an Expression but received: {bbox}"
+                )
+            bbox = Expression._to_expression(bbox).cast(DataType.fixed_size_list(DataType.uint64(), 4))
+        assert isinstance(bbox, Expression)
+        return Expression._from_pyexpr(self._expr.image_crop(bbox._expr))
