@@ -16,7 +16,6 @@ from typing import Any
 
 import fsspec
 import pyarrow as pa
-from fsspec.implementations.http import HTTPFileSystem
 from fsspec.registry import get_filesystem_class
 from loguru import logger
 from pyarrow.fs import (
@@ -26,7 +25,8 @@ from pyarrow.fs import (
     _resolve_filesystem_and_path,
 )
 
-from daft.datasources import ParquetSourceInfo, SourceInfo
+from daft.daft import FileFormat, FileFormatConfig, FileInfos, ParquetSourceConfig
+from daft.table import Table
 
 _CACHED_FSES: dict[str, FileSystem] = {}
 
@@ -253,6 +253,8 @@ def _resolve_path_and_filesystem(
 
 def _is_http_fs(fs: FileSystem) -> bool:
     """Returns whether the provided pyarrow filesystem is an HTTP filesystem."""
+    from fsspec.implementations.http import HTTPFileSystem
+
     return (
         isinstance(fs, PyFileSystem)
         and isinstance(fs.handler, FSSpecHandler)
@@ -292,9 +294,9 @@ def _path_is_glob(path: str) -> bool:
 
 def glob_path_with_stats(
     path: str,
-    source_info: SourceInfo | None,
+    file_format_config: FileFormatConfig | None,
     fs: fsspec.AbstractFileSystem,
-) -> list[ListingInfo]:
+) -> FileInfos:
     """Glob a path, returning a list ListingInfo."""
     protocol = get_protocol_from_path(path)
 
@@ -306,13 +308,11 @@ def glob_path_with_stats(
         for path, details in globbed_data.items():
             path = _ensure_path_protocol(protocol, path)
             filepaths_to_infos[path]["size"] = details["size"]
-            filepaths_to_infos[path]["type"] = details["type"]
 
     elif fs.isfile(path):
         file_info = fs.info(path)
 
         filepaths_to_infos[path]["size"] = file_info["size"]
-        filepaths_to_infos[path]["type"] = file_info["type"]
 
     elif fs.isdir(path):
         files_info = fs.ls(path, detail=True)
@@ -321,20 +321,34 @@ def glob_path_with_stats(
             path = file_info["name"]
             path = _ensure_path_protocol(protocol, path)
             filepaths_to_infos[path]["size"] = file_info["size"]
-            filepaths_to_infos[path]["type"] = file_info["type"]
 
     else:
         raise FileNotFoundError(f"File or directory not found: {path}")
 
     # Set number of rows if available.
-    if isinstance(source_info, ParquetSourceInfo):
-        parquet_metadatas = ThreadPoolExecutor().map(_get_parquet_metadata_single, filepaths_to_infos.keys())
-        for path, parquet_metadata in zip(filepaths_to_infos.keys(), parquet_metadatas):
-            filepaths_to_infos[path]["rows"] = parquet_metadata.num_rows
+    if file_format_config is not None and file_format_config.file_format() == FileFormat.Parquet:
+        config = file_format_config.config
+        assert isinstance(config, ParquetSourceConfig)
+        if config.use_native_downloader:
+            parquet_statistics = Table.read_parquet_statistics(
+                list(filepaths_to_infos.keys()), config.io_config
+            ).to_pydict()
+            for path, num_rows in zip(parquet_statistics["uris"], parquet_statistics["row_count"]):
+                filepaths_to_infos[path]["rows"] = num_rows
+        else:
+            parquet_metadatas = ThreadPoolExecutor().map(_get_parquet_metadata_single, filepaths_to_infos.keys())
+            for path, parquet_metadata in zip(filepaths_to_infos.keys(), parquet_metadatas):
+                filepaths_to_infos[path]["rows"] = parquet_metadata.num_rows
 
-    return [
-        ListingInfo(path=_ensure_path_protocol(protocol, path), **infos) for path, infos in filepaths_to_infos.items()
-    ]
+    file_paths = []
+    file_sizes = []
+    num_rows = []
+    for path, infos in filepaths_to_infos.items():
+        file_paths.append(_ensure_path_protocol(protocol, path))
+        file_sizes.append(infos.get("size"))
+        num_rows.append(infos.get("rows"))
+
+    return FileInfos.from_infos(file_paths=file_paths, file_sizes=file_sizes, num_rows=num_rows)
 
 
 def _get_parquet_metadata_single(path: str) -> pa.parquet.FileMetadata:
