@@ -2,7 +2,7 @@ from __future__ import annotations
 
 from loguru import logger
 
-from daft import resource_request
+from daft.daft import PartitionScheme, ResourceRequest
 from daft.expressions import ExpressionsProjection, col
 from daft.internal.rule import Rule
 from daft.logical.logical_plan import (
@@ -14,7 +14,6 @@ from daft.logical.logical_plan import (
     LocalAggregate,
     LocalLimit,
     LogicalPlan,
-    PartitionScheme,
     Projection,
     Repartition,
     Sort,
@@ -291,7 +290,7 @@ class DropRepartition(Rule[LogicalPlan]):
         """
         if (
             parent.partition_spec() == child.partition_spec()
-            and parent.partition_spec().scheme != PartitionScheme.RANGE
+            and parent.partition_spec().scheme != PartitionScheme.Range
         ):
             logger.debug(f"Dropping Repartition due to same spec: {parent} ")
             return child
@@ -333,10 +332,9 @@ class PushDownClausesIntoScan(Rule[LogicalPlan]):
             schema=child._schema,
             predicate=child._predicate,
             columns=child._column_names,
-            source_info=child._source_info,
+            file_format_config=child._file_format_config,
             fs=child._fs,
             filepaths_child=child._filepaths_child,
-            filepaths_column_name=child._filepaths_column_name,
             limit_rows=new_limit_rows,
         )
         return new_scan
@@ -357,10 +355,9 @@ class PushDownClausesIntoScan(Rule[LogicalPlan]):
             schema=child._schema,
             predicate=child._predicate,
             columns=ordered_required_columns,
-            source_info=child._source_info,
+            file_format_config=child._file_format_config,
             fs=child._fs,
             filepaths_child=child._filepaths_child,
-            filepaths_column_name=child._filepaths_column_name,
         )
         if any(not e._is_column() for e in parent._projection):
             return parent.copy_with_new_children([new_scan])
@@ -404,7 +401,7 @@ class FoldProjections(Rule[LogicalPlan]):
             return Projection(
                 grandchild,
                 ExpressionsProjection(new_exprs),
-                custom_resource_request=resource_request.ResourceRequest.max_resources(
+                custom_resource_request=ResourceRequest.max_resources(
                     [parent.resource_request(), child.resource_request()]
                 ),
             )
