@@ -1,10 +1,23 @@
 from __future__ import annotations
 
-from typing import Iterator
+import sys
+from typing import TYPE_CHECKING, Iterator
 
 from daft.daft import PyField as _PyField
 from daft.daft import PySchema as _PySchema
-from daft.datatype import DataType
+from daft.daft import read_parquet_schema as _read_parquet_schema
+from daft.datatype import DataType, TimeUnit
+
+if TYPE_CHECKING:
+    import pyarrow as pa
+
+if sys.version_info < (3, 8):
+    pass
+else:
+    pass
+
+if TYPE_CHECKING:
+    from daft.io import IOConfig
 
 
 class Field:
@@ -48,6 +61,20 @@ class Schema:
         s._schema = schema
         return s
 
+    @classmethod
+    def from_pyarrow_schema(cls, pa_schema: pa.Schema) -> Schema:
+        """Creates a Daft Schema from a PyArrow Schema
+
+        Args:
+            pa_schema (pa.Schema): PyArrow schema to convert
+
+        Returns:
+            Schema: Converted Daft schema
+        """
+        return cls._from_field_name_and_types(
+            [(pa_field.name, DataType.from_arrow_type(pa_field.type)) for pa_field in pa_schema]
+        )
+
     @classmethod
     def _from_field_name_and_types(self, fields: list[tuple[str, DataType]]) -> Schema:
         assert isinstance(fields, list), f"Expected a list of field tuples, received: {fields}"
@@ -113,3 +140,18 @@ class Schema:
     def __setstate__(self, state: bytes) -> None:
         self._schema = _PySchema.__new__(_PySchema)
         self._schema.__setstate__(state)
+
+    @classmethod
+    def from_parquet(
+        cls,
+        path: str,
+        io_config: IOConfig | None = None,
+        coerce_int96_timestamp_unit: TimeUnit = TimeUnit.ns(),
+    ) -> Schema:
+        return Schema._from_pyschema(
+            _read_parquet_schema(
+                uri=path,
+                io_config=io_config,
+                coerce_int96_timestamp_unit=coerce_int96_timestamp_unit._timeunit,
+            )
+        )
