@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import sys
 import weakref
 from abc import abstractmethod
 from dataclasses import dataclass
@@ -8,9 +9,15 @@ from uuid import uuid4
 
 import pyarrow as pa
 
+from daft.datatype import TimeUnit
 from daft.logical.schema import Schema
 from daft.table import Table
 
+if sys.version_info < (3, 8):
+    pass
+else:
+    pass
+
 if TYPE_CHECKING:
     import pandas as pd
 
@@ -43,6 +50,17 @@ class TableParseCSVOptions:
     header_index: int | None = 0
 
 
+@dataclass(frozen=True)
+class TableParseParquetOptions:
+    """Options for parsing Parquet files
+
+    Args:
+        coerce_int96_timestamp_unit: TimeUnit to use when parsing Int96 fields
+    """
+
+    coerce_int96_timestamp_unit: TimeUnit = TimeUnit.ns()
+
+
 @dataclass(frozen=True)
 class PartialPartitionMetadata:
     num_rows: None | int
@@ -74,13 +92,15 @@ class PartitionSet(Generic[PartitionT]):
         merged_partition = self._get_merged_vpartition()
         return merged_partition.to_pydict()
 
-    def to_pandas(self, schema: Schema | None = None) -> pd.DataFrame:
+    def to_pandas(self, schema: Schema | None = None, cast_tensors_to_ray_tensor_dtype: bool = False) -> pd.DataFrame:
         merged_partition = self._get_merged_vpartition()
-        return merged_partition.to_pandas(schema=schema)
+        return merged_partition.to_pandas(
+            schema=schema, cast_tensors_to_ray_tensor_dtype=cast_tensors_to_ray_tensor_dtype
+        )
 
-    def to_arrow(self) -> pa.Table:
+    def to_arrow(self, cast_tensors_to_ray_tensor_dtype: bool = False) -> pa.Table:
         merged_partition = self._get_merged_vpartition()
-        return merged_partition.to_arrow()
+        return merged_partition.to_arrow(cast_tensors_to_ray_tensor_dtype)
 
     def items(self) -> list[tuple[PartID, PartitionT]]:
         """
@@ -133,6 +153,9 @@ class PartitionCacheEntry:
     def __eq__(self, other: object) -> bool:
         return isinstance(other, PartitionCacheEntry) and self.key == other.key
 
+    def __hash__(self) -> int:
+        return hash(self.key)
+
     def __repr__(self) -> str:
         return f"PartitionCacheEntry: {self.key}"
 
