@@ -7,27 +7,15 @@ from typing import Iterable, Iterator
 
 import fsspec
 import psutil
-import pyarrow as pa
 from loguru import logger
 
-from daft.datasources import SourceInfo
-from daft.execution import physical_plan, physical_plan_factory
+from daft.daft import FileFormatConfig, FileInfos, ResourceRequest
+from daft.execution import physical_plan
 from daft.execution.execution_step import Instruction, MaterializedResult, PartitionTask
 from daft.filesystem import get_filesystem_from_path, glob_path_with_stats
 from daft.internal.gpu import cuda_device_count
-from daft.internal.rule_runner import FixedPointPolicy, Once, RuleBatch, RuleRunner
-from daft.logical import logical_plan
-from daft.logical.optimizer import (
-    DropProjections,
-    DropRepartition,
-    FoldProjections,
-    PruneColumns,
-    PushDownClausesIntoScan,
-    PushDownLimit,
-    PushDownPredicates,
-)
+from daft.logical.builder import LogicalPlanBuilder
 from daft.logical.schema import Schema
-from daft.resource_request import ResourceRequest
 from daft.runners import runner_io
 from daft.runners.partitioning import (
     PartID,
@@ -79,64 +67,37 @@ class LocalPartitionSet(PartitionSet[Table]):
         pass
 
 
-class PyRunnerIO(runner_io.RunnerIO[Table]):
+class PyRunnerIO(runner_io.RunnerIO):
     def glob_paths_details(
         self,
         source_paths: list[str],
-        source_info: SourceInfo | None = None,
+        file_format_config: FileFormatConfig | None = None,
         fs: fsspec.AbstractFileSystem | None = None,
-    ) -> LocalPartitionSet:
-        all_files_infos = []
+    ) -> FileInfos:
+        file_infos = FileInfos()
         for source_path in source_paths:
             if fs is None:
                 fs = get_filesystem_from_path(source_path)
 
-            files_info = glob_path_with_stats(source_path, source_info, fs)
+            path_file_infos = glob_path_with_stats(source_path, file_format_config, fs)
 
-            if len(files_info) == 0:
+            if len(path_file_infos) == 0:
                 raise FileNotFoundError(f"No files found at {source_path}")
 
-            all_files_infos.extend(files_info)
+            file_infos.extend(path_file_infos)
 
-        partition = Table.from_pydict(
-            {
-                "path": pa.array([file_info.path for file_info in all_files_infos], type=pa.string()),
-                "size": pa.array([file_info.size for file_info in all_files_infos], type=pa.int64()),
-                "type": pa.array([file_info.type for file_info in all_files_infos], type=pa.string()),
-                "rows": pa.array([file_info.rows for file_info in all_files_infos], type=pa.int64()),
-            },
-        )
-
-        # Make sure that the schema is consistent with what we expect
-        assert (
-            partition.schema() == PyRunnerIO.FS_LISTING_SCHEMA
-        ), f"Schema should be expected: {PyRunnerIO.FS_LISTING_SCHEMA}, but received: {partition.schema()}"
-
-        pset = LocalPartitionSet(
-            {
-                # Hardcoded to 1 partition
-                0: partition,
-            }
-        )
-        return pset
+        return file_infos
 
     def get_schema_from_first_filepath(
         self,
-        listing_details_partitions: PartitionSet[Table],
-        source_info: SourceInfo,
+        file_infos: FileInfos,
+        file_format_config: FileFormatConfig,
         fs: fsspec.AbstractFileSystem | None,
     ) -> Schema:
-        # Naively retrieve the first filepath in the PartitionSet
-        nonempty_partitions = [
-            p
-            for p, p_len in zip(listing_details_partitions.values(), listing_details_partitions.len_of_partitions())
-            if p_len > 0
-        ]
-        if len(nonempty_partitions) == 0:
+        if len(file_infos) == 0:
             raise ValueError("No files to get schema from")
-        first_filepath = nonempty_partitions[0].to_pydict()[PyRunnerIO.FS_LISTING_PATH_COLUMN_NAME][0]
-
-        return runner_io.sample_schema(first_filepath, source_info, fs)
+        # Naively retrieve the first filepath in the PartitionSet
+        return runner_io.sample_schema(file_infos[0].file_path, file_format_config, fs)
 
 
 class PyRunner(Runner[Table]):
@@ -144,40 +105,15 @@ class PyRunner(Runner[Table]):
         super().__init__()
         self._use_thread_pool: bool = use_thread_pool if use_thread_pool is not None else True
 
-        self._optimizer = RuleRunner(
-            [
-                RuleBatch(
-                    "SinglePassPushDowns",
-                    Once,
-                    [
-                        DropRepartition(),
-                        PushDownPredicates(),
-                        PruneColumns(),
-                        FoldProjections(),
-                        PushDownClausesIntoScan(),
-                    ],
-                ),
-                RuleBatch(
-                    "PushDownLimitsAndRepartitions",
-                    FixedPointPolicy(3),
-                    [PushDownLimit(), DropRepartition(), DropProjections()],
-                ),
-            ]
-        )
-
         self.num_cpus = multiprocessing.cpu_count()
         self.num_gpus = cuda_device_count()
         self.bytes_memory = psutil.virtual_memory().total
 
-    def optimize(self, plan: logical_plan.LogicalPlan) -> logical_plan.LogicalPlan:
-        # From PyRunner
-        return self._optimizer.optimize(plan)
-
     def runner_io(self) -> PyRunnerIO:
         return PyRunnerIO()
 
-    def run(self, logplan: logical_plan.LogicalPlan) -> PartitionCacheEntry:
-        partitions = list(self.run_iter(logplan))
+    def run(self, builder: LogicalPlanBuilder) -> PartitionCacheEntry:
+        partitions = list(self.run_iter(builder))
 
         result_pset = LocalPartitionSet({})
         for i, partition in enumerate(partitions):
@@ -186,21 +122,26 @@ class PyRunner(Runner[Table]):
         pset_entry = self.put_partition_set_into_cache(result_pset)
         return pset_entry
 
-    def run_iter(self, logplan: logical_plan.LogicalPlan) -> Iterator[Table]:
-        logplan = self.optimize(logplan)
+    def run_iter(self, builder: LogicalPlanBuilder) -> Iterator[Table]:
+        # Optimize the logical plan.
+        builder = builder.optimize()
+        # Finalize the logical plan and get a physical plan scheduler for translating the
+        # physical plan to executable tasks.
+        plan_scheduler = builder.to_physical_plan_scheduler()
         psets = {
             key: entry.value.values()
             for key, entry in self._part_set_cache._uuid_to_partition_set.items()
             if entry.value is not None
         }
-        plan = physical_plan_factory.get_materializing_physical_plan(logplan, psets)
+        # Get executable tasks from planner.
+        tasks = plan_scheduler.to_partition_tasks(psets)
 
         with profiler("profile_PyRunner.run_{datetime.now().isoformat()}.json"):
-            partitions_gen = self._physical_plan_to_partitions(plan)
+            partitions_gen = self._physical_plan_to_partitions(tasks)
             yield from partitions_gen
 
-    def run_iter_tables(self, plan: logical_plan.LogicalPlan) -> Iterator[Table]:
-        return self.run_iter(plan)
+    def run_iter_tables(self, builder: LogicalPlanBuilder) -> Iterator[Table]:
+        return self.run_iter(builder)
 
     def _physical_plan_to_partitions(self, plan: physical_plan.MaterializedPhysicalPlan) -> Iterator[Table]:
         inflight_tasks: dict[str, PartitionTask] = dict()
@@ -213,10 +154,8 @@ class PyRunner(Runner[Table]):
 
                 # Dispatch->Await loop.
                 while True:
-
                     # Dispatch loop.
                     while True:
-
                         if next_step is None:
                             # Blocked on already dispatched tasks; await some tasks.
                             break
