@@ -12,6 +12,9 @@ import fsspec
 import pyarrow as pa
 from loguru import logger
 
+from daft.logical.builder import LogicalPlanBuilder
+from daft.planner import PhysicalPlanScheduler
+
 try:
     import ray
 except ImportError:
@@ -20,9 +23,8 @@ except ImportError:
     )
     raise
 
-from daft.datasources import SourceInfo
+from daft.daft import FileFormatConfig, FileInfos, ResourceRequest
 from daft.datatype import DataType
-from daft.execution import physical_plan_factory
 from daft.execution.execution_step import (
     FanoutInstruction,
     Instruction,
@@ -33,18 +35,6 @@ from daft.execution.execution_step import (
     SingleOutputPartitionTask,
 )
 from daft.filesystem import get_filesystem_from_path, glob_path_with_stats
-from daft.internal.rule_runner import FixedPointPolicy, Once, RuleBatch, RuleRunner
-from daft.logical import logical_plan
-from daft.logical.optimizer import (
-    DropProjections,
-    DropRepartition,
-    FoldProjections,
-    PruneColumns,
-    PushDownClausesIntoScan,
-    PushDownLimit,
-    PushDownPredicates,
-)
-from daft.resource_request import ResourceRequest
 from daft.runners import runner_io
 from daft.runners.partitioning import (
     PartID,
@@ -75,43 +65,28 @@ RAY_VERSION = tuple(int(s) for s in ray.__version__.split("."))
 
 
 @ray.remote
-def _glob_path_into_details_vpartitions(
+def _glob_path_into_file_infos(
     paths: list[str],
-    schema: Schema,
-    source_info: SourceInfo | None,
+    file_format_config: FileFormatConfig | None,
     fs: fsspec.AbstractFileSystem | None,
-) -> list[tuple[PartID, Table]]:
-    all_listing_infos = []
+) -> Table:
+    file_infos = FileInfos()
     for path in paths:
         if fs is None:
             fs = get_filesystem_from_path(path)
 
-        listing_infos = glob_path_with_stats(path, source_info, fs)
-        if len(listing_infos) == 0:
+        path_file_infos = glob_path_with_stats(path, file_format_config, fs)
+        if len(path_file_infos) == 0:
             raise FileNotFoundError(f"No files found at {path}")
-        all_listing_infos.extend(listing_infos)
+        file_infos.extend(path_file_infos)
 
-    # Hardcoded to 1 partition
-    partition = Table.from_pydict(
-        {
-            "path": pa.array([file_info.path for file_info in all_listing_infos], type=pa.string()),
-            "size": pa.array([file_info.size for file_info in all_listing_infos], type=pa.int64()),
-            "type": pa.array([file_info.type for file_info in all_listing_infos], type=pa.string()),
-            "rows": pa.array([file_info.rows for file_info in all_listing_infos], type=pa.int64()),
-        },
-    )
-    assert partition.schema() == schema, f"Schema should be expected: {schema}, but received: {partition.schema()}"
-
-    partition_ref = ray.put(partition)
-    partition_refs = [(0, partition_ref)]
-
-    return partition_refs
+    return file_infos
 
 
 @ray.remote
 def _make_ray_block_from_vpartition(partition: Table) -> RayDatasetBlock:
     try:
-        return partition.to_arrow()
+        return partition.to_arrow(cast_tensors_to_ray_tensor_dtype=True)
     except pa.ArrowInvalid:
         return partition.to_pylist()
 
@@ -145,18 +120,14 @@ def remote_len_partition(p: Table) -> int:
 
 
 @ray.remote
-def sample_schema_from_filepath_vpartition(
-    p: Table,
-    filepath_column: str,
-    source_info: SourceInfo,
+def sample_schema_from_filepath(
+    first_file_path: str,
+    file_format_config: FileFormatConfig,
     fs: fsspec.AbstractFileSystem | None,
 ) -> Schema:
-    """Ray remote function to run schema sampling on top of a Table containing filepaths"""
-    assert len(p) > 0
-
+    """Ray remote function to run schema sampling on top of a Table containing a single filepath"""
     # Currently just samples the Schema from the first file
-    first_filepath = p.to_pydict()[filepath_column][0]
-    return runner_io.sample_schema(first_filepath, source_info, fs)
+    return runner_io.sample_schema(first_file_path, file_format_config, fs)
 
 
 @dataclass
@@ -231,37 +202,30 @@ class RayPartitionSet(PartitionSet[ray.ObjectRef]):
         ray.wait([o for o in self._partitions.values()])
 
 
-class RayRunnerIO(runner_io.RunnerIO[ray.ObjectRef]):
+class RayRunnerIO(runner_io.RunnerIO):
     def glob_paths_details(
         self,
         source_paths: list[str],
-        source_info: SourceInfo | None = None,
+        file_format_config: FileFormatConfig | None = None,
         fs: fsspec.AbstractFileSystem | None = None,
-    ) -> RayPartitionSet:
-        partition_refs = ray.get(
-            _glob_path_into_details_vpartitions.remote(source_paths, RayRunnerIO.FS_LISTING_SCHEMA, source_info, fs=fs)
-        )
-        return RayPartitionSet({part_id: part for part_id, part in partition_refs})
+    ) -> FileInfos:
+        # Synchronously fetch the file infos, for now.
+        return ray.get(_glob_path_into_file_infos.remote(source_paths, file_format_config, fs=fs))
 
     def get_schema_from_first_filepath(
         self,
-        listing_details_partitions: PartitionSet[ray.ObjectRef],
-        source_info: SourceInfo,
+        file_infos: FileInfos,
+        file_format_config: FileFormatConfig,
         fs: fsspec.AbstractFileSystem | None,
     ) -> Schema:
-        nonempty_partitions: list[ray.ObjectRef] = [
-            p
-            for p, p_len in zip(listing_details_partitions.values(), listing_details_partitions.len_of_partitions())
-            if p_len > 0
-        ]
-        if len(nonempty_partitions) == 0:
+        if len(file_infos) == 0:
             raise ValueError("No files to get schema from")
-        partition: ray.ObjectRef = nonempty_partitions[0]
+        # Naively retrieve the first filepath in the file info table.
+        first_path = file_infos[0].file_path
         return ray.get(
-            sample_schema_from_filepath_vpartition.remote(
-                partition,
-                RayRunnerIO.FS_LISTING_PATH_COLUMN_NAME,
-                source_info,
+            sample_schema_from_filepath.remote(
+                first_path,
+                file_format_config,
                 fs,
             )
         )
@@ -285,6 +249,10 @@ class RayRunnerIO(runner_io.RunnerIO[ray.ObjectRef]):
             )
             arrow_schema = ds.schema(fetch_if_missing=True)
 
+            # Ray 2.5.0 broke the API by using its own `ray.data.dataset.Schema` instead of PyArrow schemas
+            if RAY_VERSION >= (2, 5, 0):
+                arrow_schema = pa.schema({name: t for name, t in zip(arrow_schema.names, arrow_schema.types)})
+
         daft_schema = Schema._from_field_name_and_types(
             [(arrow_field.name, DataType.from_arrow_type(arrow_field.type)) for arrow_field in arrow_schema]
         )
@@ -403,7 +371,6 @@ class Scheduler:
         self.results_by_df: dict[str, Queue] = defaultdict(Queue)
 
     def next(self, result_uuid: str) -> ray.ObjectRef | StopIteration:
-
         # Case: thread is terminated and no longer exists.
         # Should only be hit for repeated calls to next() after StopIteration.
         if result_uuid not in self.threads_by_df:
@@ -421,16 +388,15 @@ class Scheduler:
 
     def run_plan(
         self,
-        plan: logical_plan.LogicalPlan,
+        plan_scheduler: PhysicalPlanScheduler,
         psets: dict[str, ray.ObjectRef],
         result_uuid: str,
     ) -> None:
-
         t = threading.Thread(
             target=self._run_plan,
             name=result_uuid,
             kwargs={
-                "plan": plan,
+                "plan_scheduler": plan_scheduler,
                 "psets": psets,
                 "result_uuid": result_uuid,
             },
@@ -440,13 +406,14 @@ class Scheduler:
 
     def _run_plan(
         self,
-        plan: logical_plan.LogicalPlan,
+        plan_scheduler: PhysicalPlanScheduler,
         psets: dict[str, ray.ObjectRef],
         result_uuid: str,
     ) -> None:
         from loguru import logger
 
-        phys_plan = physical_plan_factory.get_materializing_physical_plan(plan, psets)
+        # Get executable tasks from plan scheduler.
+        tasks = plan_scheduler.to_partition_tasks(psets)
 
         # Note: For autoscaling clusters, we will probably want to query cores dynamically.
         # Keep in mind this call takes about 0.3ms.
@@ -464,12 +431,10 @@ class Scheduler:
         )
         with profiler(profile_filename):
             try:
-                next_step = next(phys_plan)
+                next_step = next(tasks)
 
                 while True:  # Loop: Dispatch -> await.
-
                     while True:  # Loop: Dispatch (get tasks -> batch dispatch).
-
                         tasks_to_dispatch: list[PartitionTask] = []
 
                         dispatches_allowed = max_inflight_tasks - len(inflight_tasks)
@@ -477,7 +442,6 @@ class Scheduler:
 
                         # Loop: Get a batch of tasks.
                         while len(tasks_to_dispatch) < dispatches_allowed:
-
                             if next_step is None:
                                 # Blocked on already dispatched tasks; await some tasks.
                                 break
@@ -485,7 +449,7 @@ class Scheduler:
                             elif isinstance(next_step, ray.ObjectRef):
                                 # A final result.
                                 self.results_by_df[result_uuid].put(next_step)
-                                next_step = next(phys_plan)
+                                next_step = next(tasks)
 
                             # next_step is a task.
 
@@ -498,12 +462,12 @@ class Scheduler:
                                 next_step.set_result(
                                     [RayMaterializedResult(partition) for partition in next_step.inputs]
                                 )
-                                next_step = next(phys_plan)
+                                next_step = next(tasks)
 
                             else:
                                 # Add the task to the batch.
                                 tasks_to_dispatch.append(next_step)
-                                next_step = next(phys_plan)
+                                next_step = next(tasks)
 
                         # Dispatch the batch of tasks.
                         logger.debug(
@@ -561,7 +525,7 @@ class Scheduler:
                     )
 
                     if next_step is None:
-                        next_step = next(phys_plan)
+                        next_step = next(tasks)
 
             except StopIteration as e:
                 self.results_by_df[result_uuid].put(e)
@@ -611,26 +575,6 @@ class RayRunner(Runner[ray.ObjectRef]):
         if ray.is_initialized():
             logger.warning(f"Ray has already been initialized, Daft will reuse the existing Ray context.")
         self.ray_context = ray.init(address=address, ignore_reinit_error=True)
-        self._optimizer = RuleRunner(
-            [
-                RuleBatch(
-                    "SinglePassPushDowns",
-                    Once,
-                    [
-                        DropRepartition(),
-                        PushDownPredicates(),
-                        PruneColumns(),
-                        FoldProjections(),
-                        PushDownClausesIntoScan(),
-                    ],
-                ),
-                RuleBatch(
-                    "PushDownLimitsAndRepartitions",
-                    FixedPointPolicy(3),
-                    [PushDownLimit(), DropRepartition(), DropProjections()],
-                ),
-            ]
-        )
 
         if isinstance(self.ray_context, ray.client_builder.ClientContext):
             # Run scheduler remotely if the cluster is connected remotely.
@@ -642,8 +586,12 @@ class RayRunner(Runner[ray.ObjectRef]):
                 max_task_backlog=max_task_backlog,
             )
 
-    def run_iter(self, plan: logical_plan.LogicalPlan) -> Iterator[ray.ObjectRef]:
-        plan = self.optimize(plan)
+    def run_iter(self, builder: LogicalPlanBuilder) -> Iterator[ray.ObjectRef]:
+        # Optimize the logical plan.
+        builder = builder.optimize()
+        # Finalize the logical plan and get a physical plan scheduler for translating the
+        # physical plan to executable tasks.
+        plan_scheduler = builder.to_physical_plan_scheduler()
 
         psets = {
             key: entry.value.values()
@@ -654,7 +602,7 @@ class RayRunner(Runner[ray.ObjectRef]):
         if isinstance(self.ray_context, ray.client_builder.ClientContext):
             ray.get(
                 self.scheduler_actor.run_plan.remote(
-                    plan=plan,
+                    plan_scheduler=plan_scheduler,
                     psets=psets,
                     result_uuid=result_uuid,
                 )
@@ -662,7 +610,7 @@ class RayRunner(Runner[ray.ObjectRef]):
 
         else:
             self.scheduler.run_plan(
-                plan=plan,
+                plan_scheduler=plan_scheduler,
                 psets=psets,
                 result_uuid=result_uuid,
             )
@@ -677,14 +625,14 @@ class RayRunner(Runner[ray.ObjectRef]):
                 return
             yield result
 
-    def run_iter_tables(self, plan: logical_plan.LogicalPlan) -> Iterator[Table]:
-        for ref in self.run_iter(plan):
+    def run_iter_tables(self, builder: LogicalPlanBuilder) -> Iterator[Table]:
+        for ref in self.run_iter(builder):
             yield ray.get(ref)
 
-    def run(self, plan: logical_plan.LogicalPlan) -> PartitionCacheEntry:
+    def run(self, builder: LogicalPlanBuilder) -> PartitionCacheEntry:
         result_pset = RayPartitionSet({})
 
-        partitions_iter = self.run_iter(plan)
+        partitions_iter = self.run_iter(builder)
 
         for i, partition in enumerate(partitions_iter):
             result_pset.set_partition(i, partition)
@@ -699,9 +647,6 @@ class RayRunner(Runner[ray.ObjectRef]):
 
         return self._part_set_cache.put_partition_set(pset=pset)
 
-    def optimize(self, plan: logical_plan.LogicalPlan) -> logical_plan.LogicalPlan:
-        return self._optimizer.optimize(plan)
-
     def runner_io(self) -> RayRunnerIO:
         return RayRunnerIO()
 
