@@ -14,7 +14,8 @@ mod count;
 mod date;
 mod filter;
 mod float;
-mod full;
+pub mod from_arrow;
+pub mod full;
 mod get;
 pub(crate) mod groups;
 mod hash;
@@ -31,12 +32,15 @@ mod search_sorted;
 mod sort;
 mod sum;
 mod take;
+pub(crate) mod tensor;
 mod utf8;
 
 pub use sort::{build_multi_array_bicompare, build_multi_array_compare};
 
 use common_error::DaftResult;
 
+use crate::count_mode::CountMode;
+
 pub trait DaftCompare<Rhs> {
     type Output;
 
@@ -92,8 +96,8 @@ pub trait IntoGroups {
 
 pub trait DaftCountAggable {
     type Output;
-    fn count(&self) -> Self::Output;
-    fn grouped_count(&self, groups: &GroupIndices) -> Self::Output;
+    fn count(&self, mode: CountMode) -> Self::Output;
+    fn grouped_count(&self, groups: &GroupIndices, mode: CountMode) -> Self::Output;
 }
 
 pub trait DaftSumAggable {
