@@ -1,8 +1,16 @@
 use daft_core::{
-    datatypes::DataType, datatypes::Field, schema::Schema, utils::supertype::try_get_supertype,
+    count_mode::CountMode,
+    datatypes::DataType,
+    datatypes::{Field, FieldID},
+    schema::Schema,
+    utils::supertype::try_get_supertype,
 };
 
-use crate::{functions::FunctionEvaluator, lit};
+use crate::{
+    functions::FunctionEvaluator,
+    lit,
+    optimization::{get_required_columns, requires_computation},
+};
 
 use common_error::{DaftError, DaftResult};
 
@@ -14,9 +22,9 @@ use std::{
 
 use super::functions::FunctionExpr;
 
-type ExprRef = Arc<Expr>;
+pub type ExprRef = Arc<Expr>;
 
-#[derive(Debug, Clone, Serialize, Deserialize, PartialEq)]
+#[derive(Debug, Clone, Serialize, Deserialize, PartialEq, Eq, Hash)]
 pub enum Expr {
     Alias(ExprRef, Arc<str>),
     Agg(AggExpr),
@@ -41,9 +49,9 @@ pub enum Expr {
     },
 }
 
-#[derive(Debug, Clone, Serialize, Deserialize, PartialEq)]
+#[derive(Debug, Clone, Serialize, Deserialize, PartialEq, Eq, Hash)]
 pub enum AggExpr {
-    Count(ExprRef),
+    Count(ExprRef, CountMode),
     Sum(ExprRef),
     Mean(ExprRef),
     Min(ExprRef),
@@ -68,15 +76,54 @@ impl AggExpr {
     pub fn name(&self) -> DaftResult<&str> {
         use AggExpr::*;
         match self {
-            Count(expr) | Sum(expr) | Mean(expr) | Min(expr) | Max(expr) | List(expr)
+            Count(expr, ..)
+            | Sum(expr)
+            | Mean(expr)
+            | Min(expr)
+            | Max(expr)
+            | List(expr)
             | Concat(expr) => expr.name(),
         }
     }
 
+    pub fn semantic_id(&self, schema: &Schema) -> FieldID {
+        use AggExpr::*;
+        match self {
+            Count(expr, mode) => {
+                let child_id = expr.semantic_id(schema);
+                FieldID::new(format!("{child_id}.local_count({mode})"))
+            }
+            Sum(expr) => {
+                let child_id = expr.semantic_id(schema);
+                FieldID::new(format!("{child_id}.local_sum()"))
+            }
+            Mean(expr) => {
+                let child_id = expr.semantic_id(schema);
+                FieldID::new(format!("{child_id}.local_mean()"))
+            }
+            Min(expr) => {
+                let child_id = expr.semantic_id(schema);
+                FieldID::new(format!("{child_id}.local_min()"))
+            }
+            Max(expr) => {
+                let child_id = expr.semantic_id(schema);
+                FieldID::new(format!("{child_id}.local_max()"))
+            }
+            List(expr) => {
+                let child_id = expr.semantic_id(schema);
+                FieldID::new(format!("{child_id}.local_list()"))
+            }
+            Concat(expr) => {
+                let child_id = expr.semantic_id(schema);
+                FieldID::new(format!("{child_id}.local_concat()"))
+            }
+        }
+    }
+
     pub fn to_field(&self, schema: &Schema) -> DaftResult<Field> {
         use AggExpr::*;
         match self {
-            Count(expr) => {
+            Count(expr, ..) => {
                 let field = expr.to_field(schema)?;
                 Ok(Field::new(field.name.as_str(), DataType::UInt64))
             }
@@ -153,7 +200,7 @@ impl AggExpr {
     pub fn from_name_and_child_expr(name: &str, child: &Expr) -> DaftResult<AggExpr> {
         use AggExpr::*;
         match name {
-            "count" => Ok(Count(child.clone().into())),
+            "count" => Ok(Count(child.clone().into(), CountMode::Valid)),
             "sum" => Ok(Sum(child.clone().into())),
             "mean" => Ok(Mean(child.clone().into())),
             "min" => Ok(Min(child.clone().into())),
@@ -190,8 +237,8 @@ impl Expr {
         Expr::Cast(self.clone().into(), dtype.clone())
     }
 
-    pub fn count(&self) -> Self {
-        Expr::Agg(AggExpr::Count(self.clone().into()))
+    pub fn count(&self, mode: CountMode) -> Self {
+        Expr::Agg(AggExpr::Count(self.clone().into(), mode))
     }
 
     pub fn sum(&self) -> Self {
@@ -226,10 +273,96 @@ impl Expr {
         Expr::IsNull(self.clone().into())
     }
 
+    pub fn eq(&self, other: &Self) -> Self {
+        binary_op(Operator::Eq, self, other)
+    }
+
+    pub fn not_eq(&self, other: &Self) -> Self {
+        binary_op(Operator::NotEq, self, other)
+    }
+
     pub fn and(&self, other: &Self) -> Self {
         binary_op(Operator::And, self, other)
     }
 
+    pub fn or(&self, other: &Self) -> Self {
+        binary_op(Operator::Or, self, other)
+    }
+
+    pub fn lt(&self, other: &Self) -> Self {
+        binary_op(Operator::Lt, self, other)
+    }
+
+    pub fn lt_eq(&self, other: &Self) -> Self {
+        binary_op(Operator::LtEq, self, other)
+    }
+
+    pub fn gt(&self, other: &Self) -> Self {
+        binary_op(Operator::Gt, self, other)
+    }
+
+    pub fn gt_eq(&self, other: &Self) -> Self {
+        binary_op(Operator::GtEq, self, other)
+    }
+
+    pub fn semantic_id(&self, schema: &Schema) -> FieldID {
+        use Expr::*;
+        match self {
+            // Base case - anonymous column reference.
+            // Look up the column name in the provided schema and get its field ID.
+            Column(name) => FieldID::new(&**name),
+
+            // Base case - literal.
+            Literal(value) => FieldID::new(format!("Literal({value:?})")),
+
+            // Recursive cases.
+            Cast(expr, dtype) => {
+                let child_id = expr.semantic_id(schema);
+                FieldID::new(format!("{child_id}.cast({dtype})"))
+            }
+            Not(expr) => {
+                let child_id = expr.semantic_id(schema);
+                FieldID::new(format!("{child_id}.not()"))
+            }
+            IsNull(expr) => {
+                let child_id = expr.semantic_id(schema);
+                FieldID::new(format!("{child_id}.is_null()"))
+            }
+            Function { func, inputs } => {
+                let inputs = inputs
+                    .iter()
+                    .map(|expr| expr.semantic_id(schema).id.to_string())
+                    .collect::<Vec<String>>()
+                    .join(", ");
+                // TODO: check for function idempotency here.
+                FieldID::new(format!("Function_{func:?}({inputs})"))
+            }
+            BinaryOp { op, left, right } => {
+                let left_id = left.semantic_id(schema);
+                let right_id = right.semantic_id(schema);
+                // TODO: check for symmetry here.
+                FieldID::new(format!("({left_id} {op} {right_id})"))
+            }
+
+            IfElse {
+                if_true,
+                if_false,
+                predicate,
+            } => {
+                let if_true = if_true.semantic_id(schema);
+                let if_false = if_false.semantic_id(schema);
+                let predicate = predicate.semantic_id(schema);
+                FieldID::new(format!("({if_true} if {predicate} else {if_false})"))
+            }
+
+            // Alias: ID does not change.
+            Alias(expr, ..) => expr.semantic_id(schema),
+
+            // Agg: Separate path.
+            Agg(agg_expr) => agg_expr.semantic_id(schema),
+        }
+    }
+
     pub fn to_field(&self, schema: &Schema) -> DaftResult<Field> {
         use Expr::*;
         match self {
@@ -256,21 +389,8 @@ impl Expr {
                 match op {
                     // Logical operations
                     Operator::And | Operator::Or | Operator::Xor => {
-                        match try_get_supertype(&left_field.dtype, &right_field.dtype) {
-                            Ok(DataType::Boolean) => Ok(Field::new(
-                                left_field.name.as_str(),
-                                DataType::Boolean,
-                            )),
-                            #[cfg(feature = "python")]
-                            Ok(DataType::Python) => Ok(Field::new(
-                                left_field.name.as_str(),
-                                DataType::Boolean,
-                            )),
-                            Ok(other_stype) => Err(DaftError::TypeError(format!(
-                                "Expected boolean supertype arguments for {op} but received {left_field} {op} {right_field} with supertype {other_stype}",
-                            ))),
-                            Err(_) => Err(DaftError::TypeError(format!("Expected left and right arguments to be castable to the same supertype for comparison {op}, but received {left_field} and {right_field}"))),
-                        }
+                        let result_type = left_field.dtype.logical_op(&right_field.dtype)?;
+                        Ok(Field::new(left_field.name.as_str(), result_type))
                     }
 
                     // Comparison operations
@@ -280,22 +400,12 @@ impl Expr {
                     | Operator::NotEq
                     | Operator::LtEq
                     | Operator::GtEq => {
-                        // TODO: [ISSUE-688] Make Binary type comparable
-                        if left_field.dtype == DataType::Binary
-                            || right_field.dtype == DataType::Binary
-                        {
-                            return Err(DaftError::TypeError(format!(
-                                "Binary types cannot be compared: {left_field} {op} {right_field}",
-                            )));
-                        }
-                        match try_get_supertype(&left_field.dtype, &right_field.dtype) {
-                            Ok(_) => Ok(Field::new(
-                                left.to_field(schema)?.name.as_str(),
-                                DataType::Boolean,
-                            )),
-                            Err(_) => Err(DaftError::TypeError(format!("Expected left and right arguments to be castable to the same supertype for comparison {op}, but received {left_field} and {right_field}"))),
-                        }
+                        let (result_type, _comp_type) =
+                            left_field.dtype.comparison_op(&right_field.dtype)?;
+                        Ok(Field::new(left_field.name.as_str(), result_type))
                     }
+
+                    // Arithmetic operations
                     Operator::Plus => {
                         let result_type = (&left_field.dtype + &right_field.dtype)?;
                         Ok(Field::new(left_field.name.as_str(), result_type))
@@ -365,6 +475,19 @@ impl Expr {
     pub fn get_type(&self, schema: &Schema) -> DaftResult<DataType> {
         Ok(self.to_field(schema)?.dtype)
     }
+
+    pub fn input_mapping(&self) -> Option<String> {
+        let required_columns = get_required_columns(self);
+        let requires_computation = requires_computation(self);
+
+        // Return the required column only if:
+        //   1. There is only one required column
+        //   2. No computation is run on this required column
+        match (&required_columns[..], requires_computation) {
+            ([required_col], false) => Some(required_col.clone()),
+            _ => None,
+        }
+    }
 }
 
 impl Display for Expr {
@@ -417,7 +540,7 @@ impl Display for AggExpr {
     fn fmt(&self, f: &mut Formatter<'_>) -> Result {
         use AggExpr::*;
         match self {
-            Count(expr) => write!(f, "count({expr})"),
+            Count(expr, mode) => write!(f, "count({expr}, {mode})"),
             Sum(expr) => write!(f, "sum({expr})"),
             Mean(expr) => write!(f, "mean({expr})"),
             Min(expr) => write!(f, "min({expr})"),
@@ -429,7 +552,7 @@ impl Display for AggExpr {
 }
 
 /// Based on Polars first class operators: https://github.com/pola-rs/polars/blob/master/polars/polars-lazy/polars-plan/src/dsl/expr.rs
-#[derive(Debug, Copy, Clone, PartialEq, Eq, Serialize, Deserialize)]
+#[derive(Debug, Copy, Clone, PartialEq, Eq, Serialize, Deserialize, Hash)]
 pub enum Operator {
     Eq,
     NotEq,
