@@ -1,18 +1,21 @@
 mod explode;
 mod join;
+mod lengths;
 
 use explode::ExplodeEvaluator;
 use join::JoinEvaluator;
+use lengths::LengthsEvaluator;
 use serde::{Deserialize, Serialize};
 
 use crate::Expr;
 
 use super::FunctionEvaluator;
 
-#[derive(Debug, Clone, Serialize, Deserialize, PartialEq)]
+#[derive(Debug, Clone, Serialize, Deserialize, PartialEq, Eq, Hash)]
 pub enum ListExpr {
     Explode,
     Join,
+    Lengths,
 }
 
 impl ListExpr {
@@ -22,6 +25,7 @@ impl ListExpr {
         match self {
             Explode => &ExplodeEvaluator {},
             Join => &JoinEvaluator {},
+            Lengths => &LengthsEvaluator {},
         }
     }
 }
@@ -39,3 +43,10 @@ pub fn join(input: &Expr, delimiter: &Expr) -> Expr {
         inputs: vec![input.clone(), delimiter.clone()],
     }
 }
+
+pub fn lengths(input: &Expr) -> Expr {
+    Expr::Function {
+        func: super::FunctionExpr::List(ListExpr::Lengths),
+        inputs: vec![input.clone()],
+    }
+}
