@@ -1,16 +1,20 @@
 use std::ops::Range;
-use std::path::PathBuf;
 
 use async_trait::async_trait;
 use bytes::Bytes;
 use futures::stream::{BoxStream, Stream};
 use futures::StreamExt;
+use tokio::sync::OwnedSemaphorePermit;
 
 use crate::local::{collect_file, LocalFile};
 
-pub(crate) enum GetResult {
+pub enum GetResult {
     File(LocalFile),
-    Stream(BoxStream<'static, super::Result<Bytes>>, Option<usize>),
+    Stream(
+        BoxStream<'static, super::Result<Bytes>>,
+        Option<usize>,
+        Option<OwnedSemaphorePermit>,
+    ),
 }
 
 async fn collect_bytes<S>(mut stream: S, size_hint: Option<usize>) -> super::Result<Bytes>
@@ -41,15 +45,41 @@ impl GetResult {
         use GetResult::*;
         match self {
             File(f) => collect_file(f).await,
-            Stream(stream, size) => collect_bytes(stream, size).await,
+            Stream(stream, size, _permit) => collect_bytes(stream, size).await,
         }
     }
 }
 
+#[derive(Debug)]
+pub enum FileType {
+    File,
+    Directory,
+}
+#[derive(Debug)]
+pub struct FileMetadata {
+    pub filepath: String,
+    pub size: Option<u64>,
+    pub filetype: FileType,
+}
+#[derive(Debug)]
+pub struct LSResult {
+    pub files: Vec<FileMetadata>,
+    pub continuation_token: Option<String>,
+}
+
 #[async_trait]
 pub(crate) trait ObjectSource: Sync + Send {
     async fn get(&self, uri: &str, range: Option<Range<usize>>) -> super::Result<GetResult>;
     async fn get_range(&self, uri: &str, range: Range<usize>) -> super::Result<GetResult> {
         self.get(uri, Some(range)).await
     }
+    async fn get_size(&self, uri: &str) -> super::Result<usize>;
+    async fn ls(
+        &self,
+        path: &str,
+        delimiter: Option<&str>,
+        continuation_token: Option<&str>,
+    ) -> super::Result<LSResult>;
+
+    // async fn iter_dir(&self, path: &str, limit: Option<usize>) -> super::Result<Box<dyn Stream<Item = super::Result<LSResult>>>>;
 }
