@@ -3,16 +3,16 @@
 use std::collections::HashSet;
 use std::fmt::{Display, Formatter, Result};
 
+use daft_core::array::ops::full::FullNull;
 use num_traits::ToPrimitive;
 
 use daft_core::array::ops::GroupIndices;
 
 use common_error::{DaftError, DaftResult};
-use daft_core::datatypes::logical::LogicalArray;
-use daft_core::datatypes::{BooleanType, DataType, Field, UInt64Array};
+use daft_core::datatypes::{BooleanArray, DataType, Field, UInt64Array};
 use daft_core::schema::{Schema, SchemaRef};
 use daft_core::series::{IntoSeries, Series};
-use daft_core::{with_match_daft_logical_types, with_match_physical_daft_types};
+
 use daft_dsl::functions::FunctionEvaluator;
 use daft_dsl::{col, null_lit, AggExpr, Expr};
 #[cfg(feature = "python")]
@@ -72,15 +72,8 @@ impl Table {
             Some(schema) => {
                 let mut columns: Vec<Series> = Vec::with_capacity(schema.names().len());
                 for (field_name, field) in schema.fields.iter() {
-                    if field.dtype.is_logical() {
-                        with_match_daft_logical_types!(field.dtype, |$T| {
-                            columns.push(LogicalArray::<$T>::empty(field_name, &field.dtype).into_series())
-                        })
-                    } else {
-                        with_match_physical_daft_types!(field.dtype, |$T| {
-                            columns.push(DataArray::<$T>::empty(field_name, &field.dtype).into_series())
-                        })
-                    }
+                    let series = Series::empty(field_name, &field.dtype);
+                    columns.push(series)
                 }
                 Ok(Table { schema, columns })
             }
@@ -202,7 +195,7 @@ impl Table {
             )));
         }
 
-        let mask = mask.downcast::<BooleanType>().unwrap();
+        let mask = mask.downcast::<BooleanArray>().unwrap();
         let new_series: DaftResult<Vec<_>> = self.columns.iter().map(|s| s.filter(mask)).collect();
         Ok(Table {
             schema: self.schema.clone(),
@@ -274,7 +267,7 @@ impl Table {
     ) -> DaftResult<Series> {
         use daft_dsl::AggExpr::*;
         match agg_expr {
-            Count(expr) => Series::count(&self.eval_expression(expr)?, groups),
+            Count(expr, mode) => Series::count(&self.eval_expression(expr)?, groups, *mode),
             Sum(expr) => Series::sum(&self.eval_expression(expr)?, groups),
             Mean(expr) => Series::mean(&self.eval_expression(expr)?, groups),
             Min(expr) => Series::min(&self.eval_expression(expr)?, groups),
