@@ -13,14 +13,15 @@ import pyarrow as pa
 import pyarrow.parquet as papq
 import pytest
 from fsspec.implementations.local import LocalFileSystem
-from ray.data.extensions import ArrowTensorArray
+from ray.data.extensions import ArrowTensorArray, TensorArray
 
 import daft
 from daft.api_annotations import APITypeError
 from daft.context import get_context
 from daft.dataframe import DataFrame
 from daft.datatype import DataType
-from tests.conftest import UuidType
+from daft.utils import pyarrow_supports_fixed_shape_tensor
+from tests.conftest import *
 
 ARROW_VERSION = tuple(int(s) for s in pa.__version__.split(".") if s.isnumeric())
 
@@ -168,39 +169,38 @@ def test_create_dataframe_arrow(valid_data: list[dict[str, float]], multiple) ->
 
 def test_create_dataframe_arrow_tensor_ray(valid_data: list[dict[str, float]]) -> None:
     pydict = {k: [item[k] for item in valid_data] for k in valid_data[0].keys()}
-    ata = ArrowTensorArray.from_numpy(np.ones((len(valid_data), 2, 2)))
-    pydict["obj"] = ata
+    shape = (2, 2)
+    arr = np.ones((len(valid_data),) + shape)
+    ata = ArrowTensorArray.from_numpy(arr)
+    pydict["tensor"] = ata
     t = pa.Table.from_pydict(pydict)
     df = daft.from_arrow(t)
     assert set(df.column_names) == set(t.column_names)
-    # Type not natively supported, so should have Python object dtype.
-    assert df.schema()["obj"].dtype == DataType.python()
-    casted_field = t.schema.field("variety").with_type(pa.large_string())
-    expected = t.cast(t.schema.set(t.schema.get_field_index("variety"), casted_field))
+    # Tensor type should be inferred.
+    expected_tensor_dtype = DataType.tensor(DataType.float64(), shape)
+    assert df.schema()["tensor"].dtype == expected_tensor_dtype
+    casted_variety = t.schema.field("variety").with_type(pa.large_string())
+    schema = t.schema.set(t.schema.get_field_index("variety"), casted_variety)
+    expected = t.cast(schema)
     # Check roundtrip.
-    assert df.to_arrow() == expected
+    assert df.to_arrow(True) == expected
 
 
 @pytest.mark.skipif(
-    ARROW_VERSION < (12, 0, 0),
+    not pyarrow_supports_fixed_shape_tensor(),
     reason=f"Arrow version {ARROW_VERSION} doesn't support the canonical tensor extension type.",
 )
-@pytest.mark.skipif(
-    get_context().runner_config.name == "ray",
-    reason="Pickling canonical tensor extension type is not supported by pyarrow",
-)
 def test_create_dataframe_arrow_tensor_canonical(valid_data: list[dict[str, float]]) -> None:
     pydict = {k: [item[k] for item in valid_data] for k in valid_data[0].keys()}
-    dtype = pa.fixed_shape_tensor(pa.int64(), (2, 2))
+    shape = (2, 2)
+    dtype = pa.fixed_shape_tensor(pa.int64(), shape)
     storage = pa.array([list(range(4 * i, 4 * (i + 1))) for i in range(len(valid_data))], pa.list_(pa.int64(), 4))
-    ata = pa.ExtensionArray.from_storage(dtype, storage)
-    pydict["obj"] = ata
+    fst = pa.ExtensionArray.from_storage(dtype, storage)
+    pydict["tensor"] = fst
     t = pa.Table.from_pydict(pydict)
     df = daft.from_arrow(t)
     assert set(df.column_names) == set(t.column_names)
-    assert df.schema()["obj"].dtype == DataType.extension(
-        "arrow.fixed_shape_tensor", DataType.from_arrow_type(dtype.storage_type), '{"shape":[2,2]}'
-    )
+    assert df.schema()["tensor"].dtype == DataType.tensor(DataType.int64(), shape)
     casted_field = t.schema.field("variety").with_type(pa.large_string())
     expected = t.cast(t.schema.set(t.schema.get_field_index("variety"), casted_field))
     # Check roundtrip.
@@ -244,7 +244,9 @@ def test_create_dataframe_arrow_py_ext_type_raises(valid_data: list[dict[str, fl
 
 def test_create_dataframe_arrow_unsupported_dtype(valid_data: list[dict[str, float]]) -> None:
     pydict = {k: [item[k] for item in valid_data] for k in valid_data[0].keys()}
-    pydict["obj"] = [decimal.Decimal("1.1") for _ in range(len(valid_data))]
+    pydict["obj"] = [
+        decimal.Decimal("12456789012345678901234567890123456789012345678901234567890") for _ in range(len(valid_data))
+    ]
     t = pa.Table.from_pydict(pydict)
     df = daft.from_arrow(t)
     assert set(df.column_names) == set(t.column_names)
@@ -288,14 +290,14 @@ def test_create_dataframe_pandas_py_object(valid_data: list[dict[str, float]]) -
 
 def test_create_dataframe_pandas_tensor(valid_data: list[dict[str, float]]) -> None:
     pydict = {k: [item[k] for item in valid_data] for k in valid_data[0].keys()}
-    pydict["obj"] = pd.Series([np.ones((2, 2)) for _ in range(len(valid_data))])
+    shape = (2, 2)
+    pydict["tensor"] = TensorArray(np.ones((len(valid_data),) + shape))
     pd_df = pd.DataFrame(pydict)
     df = daft.from_pandas(pd_df)
-    # Type not natively supported, so should have Python object dtype.
-    assert df.schema()["obj"].dtype == DataType.python()
+    assert df.schema()["tensor"].dtype == DataType.tensor(DataType.float64(), shape)
     assert set(df.column_names) == set(pd_df.columns)
     # Check roundtrip.
-    pd.testing.assert_frame_equal(df.to_pandas(), pd_df)
+    pd.testing.assert_frame_equal(df.to_pandas(cast_tensors_to_ray_tensor_dtype=True), pd_df)
 
 
 @pytest.mark.parametrize(
@@ -320,16 +322,17 @@ def test_create_dataframe_pandas_tensor(valid_data: list[dict[str, float]]) -> N
         ),
         pytest.param(
             [np.array([1]), np.array([2]), np.array([3])],
-            DataType.list("item", DataType.int64()),
+            DataType.list(DataType.int64()),
             id="numpy_1d_arrays",
         ),
-        pytest.param(pa.array([[1, 2, 3], [1, 2], [1]]), DataType.list("item", DataType.int64()), id="pa_nested"),
+        pytest.param(pa.array([[1, 2, 3], [1, 2], [1]]), DataType.list(DataType.int64()), id="pa_nested"),
         pytest.param(
             pa.chunked_array([pa.array([[1, 2, 3], [1, 2], [1]])]),
-            DataType.list("item", DataType.int64()),
+            DataType.list(DataType.int64()),
             id="pa_nested_chunked",
         ),
-        pytest.param(np.ones((3, 3)), DataType.python(), id="np_nested"),
+        pytest.param(np.ones((3, 3)), DataType.list(DataType.float64()), id="np_nested_1d"),
+        pytest.param(np.ones((3, 3, 3)), DataType.tensor(DataType.float64()), id="np_nested_nd"),
     ],
 )
 def test_load_pydict_types(data, expected_dtype):
@@ -380,7 +383,10 @@ def test_create_dataframe_multiple_csvs(valid_data: list[dict[str, float]]) -> N
         assert len(pd_df) == (len(valid_data) * 2)
 
 
-@pytest.mark.skipif(get_context().runner_config.name not in {"py"}, reason="requires PyRunner to be in use")
+@pytest.mark.skipif(
+    (get_context().runner_config.name not in {"py"}) or get_context().use_rust_planner,
+    reason="requires PyRunner and old query planner to be in use",
+)
 def test_create_dataframe_csv_custom_fs(valid_data: list[dict[str, float]]) -> None:
     with tempfile.NamedTemporaryFile("w") as f:
         header = list(valid_data[0].keys())
@@ -550,7 +556,10 @@ def test_create_dataframe_multiple_jsons(valid_data: list[dict[str, float]]) ->
         assert len(pd_df) == (len(valid_data) * 2)
 
 
-@pytest.mark.skipif(get_context().runner_config.name not in {"py"}, reason="requires PyRunner to be in use")
+@pytest.mark.skipif(
+    (get_context().runner_config.name not in {"py"}) or get_context().use_rust_planner,
+    reason="requires PyRunner and old query planner to be in use",
+)
 def test_create_dataframe_json_custom_fs(valid_data: list[dict[str, float]]) -> None:
     with tempfile.NamedTemporaryFile("w") as f:
         for data in valid_data:
@@ -635,13 +644,14 @@ def test_create_dataframe_json_specify_schema(valid_data: list[dict[str, float]]
 ###
 
 
-def test_create_dataframe_parquet(valid_data: list[dict[str, float]]) -> None:
+@pytest.mark.parametrize("use_native_downloader", [True, False])
+def test_create_dataframe_parquet(valid_data: list[dict[str, float]], use_native_downloader) -> None:
     with tempfile.NamedTemporaryFile("w") as f:
         table = pa.Table.from_pydict({col: [d[col] for d in valid_data] for col in COL_NAMES})
         papq.write_table(table, f.name)
         f.flush()
 
-        df = daft.read_parquet(f.name)
+        df = daft.read_parquet(f.name, use_native_downloader=use_native_downloader)
         assert df.column_names == COL_NAMES
 
         pd_df = df.to_pandas()
@@ -649,14 +659,32 @@ def test_create_dataframe_parquet(valid_data: list[dict[str, float]]) -> None:
         assert len(pd_df) == len(valid_data)
 
 
-def test_create_dataframe_multiple_parquets(valid_data: list[dict[str, float]]) -> None:
+@pytest.mark.parametrize("use_native_downloader", [True, False])
+def test_create_dataframe_parquet_with_filter(valid_data: list[dict[str, float]], use_native_downloader) -> None:
+    with tempfile.NamedTemporaryFile("w") as f:
+        table = pa.Table.from_pydict({col: [d[col] for d in valid_data] for col in COL_NAMES})
+        papq.write_table(table, f.name)
+        f.flush()
+
+        df = daft.read_parquet(f.name, use_native_downloader=use_native_downloader)
+        assert df.column_names == COL_NAMES
+
+        df = df.where(daft.col("sepal_length") > 4.8)
+
+        pd_df = df.to_pandas()
+        assert list(pd_df.columns) == COL_NAMES
+        assert len(pd_df) == len(valid_data) - 1
+
+
+@pytest.mark.parametrize("use_native_downloader", [True, False])
+def test_create_dataframe_multiple_parquets(valid_data: list[dict[str, float]], use_native_downloader) -> None:
     with tempfile.NamedTemporaryFile("w") as f1, tempfile.NamedTemporaryFile("w") as f2:
         for f in (f1, f2):
             table = pa.Table.from_pydict({col: [d[col] for d in valid_data] for col in COL_NAMES})
             papq.write_table(table, f.name)
             f.flush()
 
-        df = daft.read_parquet([f1.name, f2.name])
+        df = daft.read_parquet([f1.name, f2.name], use_native_downloader=use_native_downloader)
         assert df.column_names == COL_NAMES
 
         pd_df = df.to_pandas()
@@ -664,7 +692,10 @@ def test_create_dataframe_multiple_parquets(valid_data: list[dict[str, float]])
         assert len(pd_df) == (len(valid_data) * 2)
 
 
-@pytest.mark.skipif(get_context().runner_config.name not in {"py"}, reason="requires PyRunner to be in use")
+@pytest.mark.skipif(
+    (get_context().runner_config.name not in {"py"}) or get_context().use_rust_planner,
+    reason="requires PyRunner and old query planner to be in use",
+)
 def test_create_dataframe_parquet_custom_fs(valid_data: list[dict[str, float]]) -> None:
     with tempfile.NamedTemporaryFile("w") as f:
         table = pa.Table.from_pydict({col: [d[col] for d in valid_data] for col in COL_NAMES})
@@ -694,7 +725,8 @@ def test_create_dataframe_parquet_custom_fs(valid_data: list[dict[str, float]])
         assert len(pd_df) == len(valid_data)
 
 
-def test_create_dataframe_parquet_column_projection(valid_data: list[dict[str, float]]) -> None:
+@pytest.mark.parametrize("use_native_downloader", [True, False])
+def test_create_dataframe_parquet_column_projection(valid_data: list[dict[str, float]], use_native_downloader) -> None:
     with tempfile.NamedTemporaryFile("w") as f:
         table = pa.Table.from_pydict({col: [d[col] for d in valid_data] for col in COL_NAMES})
         papq.write_table(table, f.name)
@@ -702,7 +734,7 @@ def test_create_dataframe_parquet_column_projection(valid_data: list[dict[str, f
 
         col_subset = COL_NAMES[:3]
 
-        df = daft.read_parquet(f.name)
+        df = daft.read_parquet(f.name, use_native_downloader=use_native_downloader)
         df = df.select(*col_subset)
         assert df.column_names == col_subset
 
@@ -711,7 +743,8 @@ def test_create_dataframe_parquet_column_projection(valid_data: list[dict[str, f
         assert len(pd_df) == len(valid_data)
 
 
-def test_create_dataframe_parquet_specify_schema(valid_data: list[dict[str, float]]) -> None:
+@pytest.mark.parametrize("use_native_downloader", [True, False])
+def test_create_dataframe_parquet_specify_schema(valid_data: list[dict[str, float]], use_native_downloader) -> None:
     with tempfile.NamedTemporaryFile("w") as f:
         table = pa.Table.from_pydict({col: [d[col] for d in valid_data] for col in COL_NAMES})
         papq.write_table(table, f.name)
@@ -720,12 +753,13 @@ def test_create_dataframe_parquet_specify_schema(valid_data: list[dict[str, floa
         df = daft.read_parquet(
             f.name,
             schema_hints={
-                "sepal_length": DataType.float32(),
-                "sepal_width": DataType.float32(),
-                "petal_length": DataType.float32(),
-                "petal_width": DataType.float32(),
+                "sepal_length": DataType.float64(),
+                "sepal_width": DataType.float64(),
+                "petal_length": DataType.float64(),
+                "petal_width": DataType.float64(),
                 "variety": DataType.string(),
             },
+            use_native_downloader=use_native_downloader,
         )
         assert df.column_names == COL_NAMES
 
