@@ -23,17 +23,16 @@ def test_push_projection_scan_all_cols(valid_data_json_path: str, optimizer):
     # Manually construct a plan to be what we expect after optimization
     df_optimized = DataFrame(
         TabularFilesScan(
-            schema=df_unoptimized_scan.plan()._schema,
-            predicate=df_unoptimized_scan.plan()._predicate,
+            schema=df_unoptimized_scan._get_current_builder()._plan._schema,
+            predicate=df_unoptimized_scan._get_current_builder()._plan._predicate,
             columns=["sepal_length"],
-            source_info=df_unoptimized_scan.plan()._source_info,
-            fs=df_unoptimized_scan.plan()._fs,
-            filepaths_child=df_unoptimized_scan.plan()._filepaths_child,
-            filepaths_column_name=df_unoptimized_scan.plan()._filepaths_column_name,
-        )
+            file_format_config=df_unoptimized_scan._get_current_builder()._plan._file_format_config,
+            fs=df_unoptimized_scan._get_current_builder()._plan._fs,
+            filepaths_child=df_unoptimized_scan._get_current_builder()._plan._filepaths_child,
+        ).to_builder()
     )
 
-    assert_plan_eq(optimizer(df_unoptimized.plan()), df_optimized.plan())
+    assert_plan_eq(optimizer(df_unoptimized._get_current_builder()._plan), df_optimized._get_current_builder()._plan)
 
 
 def test_push_projection_scan_all_cols_alias(valid_data_json_path: str, optimizer):
@@ -44,18 +43,17 @@ def test_push_projection_scan_all_cols_alias(valid_data_json_path: str, optimize
     # Manually construct a plan to be what we expect after optimization
     df_optimized = DataFrame(
         TabularFilesScan(
-            schema=df_unoptimized_scan.plan()._schema,
-            predicate=df_unoptimized_scan.plan()._predicate,
+            schema=df_unoptimized_scan._get_current_builder()._plan._schema,
+            predicate=df_unoptimized_scan._get_current_builder()._plan._predicate,
             columns=["sepal_length"],
-            source_info=df_unoptimized_scan.plan()._source_info,
-            fs=df_unoptimized_scan.plan()._fs,
-            filepaths_child=df_unoptimized_scan.plan()._filepaths_child,
-            filepaths_column_name=df_unoptimized_scan.plan()._filepaths_column_name,
-        )
+            file_format_config=df_unoptimized_scan._get_current_builder()._plan._file_format_config,
+            fs=df_unoptimized_scan._get_current_builder()._plan._fs,
+            filepaths_child=df_unoptimized_scan._get_current_builder()._plan._filepaths_child,
+        ).to_builder()
     )
     df_optimized = df_optimized.select(col("sepal_length").alias("foo"))
 
-    assert_plan_eq(optimizer(df_unoptimized.plan()), df_optimized.plan())
+    assert_plan_eq(optimizer(df_unoptimized._get_current_builder()._plan), df_optimized._get_current_builder()._plan)
 
 
 def test_push_projection_scan_some_cols_aliases(valid_data_json_path: str, optimizer):
@@ -66,15 +64,14 @@ def test_push_projection_scan_some_cols_aliases(valid_data_json_path: str, optim
     # Manually construct a plan to be what we expect after optimization
     df_optimized = DataFrame(
         TabularFilesScan(
-            schema=df_unoptimized_scan.plan()._schema,
-            predicate=df_unoptimized_scan.plan()._predicate,
+            schema=df_unoptimized_scan._get_current_builder()._plan._schema,
+            predicate=df_unoptimized_scan._get_current_builder()._plan._predicate,
             columns=["sepal_length", "sepal_width"],
-            source_info=df_unoptimized_scan.plan()._source_info,
-            fs=df_unoptimized_scan.plan()._fs,
-            filepaths_child=df_unoptimized_scan.plan()._filepaths_child,
-            filepaths_column_name=df_unoptimized_scan.plan()._filepaths_column_name,
-        )
+            file_format_config=df_unoptimized_scan._get_current_builder()._plan._file_format_config,
+            fs=df_unoptimized_scan._get_current_builder()._plan._fs,
+            filepaths_child=df_unoptimized_scan._get_current_builder()._plan._filepaths_child,
+        ).to_builder()
     )
     df_optimized = df_optimized.select(col("sepal_length").alias("foo"), col("sepal_width") + 1)
 
-    assert_plan_eq(optimizer(df_unoptimized.plan()), df_optimized.plan())
+    assert_plan_eq(optimizer(df_unoptimized._get_current_builder()._plan), df_optimized._get_current_builder()._plan)
