@@ -33,7 +33,7 @@ def test_no_pushdown_on_modified_column(optimizer) -> None:
     ).where(col("ints") == col("modified").alias("ints_dup"))
 
     # Optimizer cannot push down the filter because it uses a column that was projected
-    assert_plan_eq(optimizer(df.plan()), df.plan())
+    assert_plan_eq(optimizer(df._get_current_builder()._plan), df._get_current_builder()._plan)
 
 
 def test_filter_pushdown_select(valid_data: list[dict[str, float]], optimizer) -> None:
@@ -41,7 +41,7 @@ def test_filter_pushdown_select(valid_data: list[dict[str, float]], optimizer) -
     unoptimized = df.select("sepal_length", "sepal_width").where(col("sepal_length") > 4.8)
     optimized = df.where(col("sepal_length") > 4.8).select("sepal_length", "sepal_width")
     assert unoptimized.column_names == ["sepal_length", "sepal_width"]
-    assert_plan_eq(optimizer(unoptimized.plan()), optimized.plan())
+    assert_plan_eq(optimizer(unoptimized._get_current_builder()._plan), optimized._get_current_builder()._plan)
 
 
 def test_filter_pushdown_select_alias(valid_data: list[dict[str, float]], optimizer) -> None:
@@ -49,7 +49,7 @@ def test_filter_pushdown_select_alias(valid_data: list[dict[str, float]], optimi
     unoptimized = df.select("sepal_length", "sepal_width").where(col("sepal_length").alias("foo") > 4.8)
     optimized = df.where(col("sepal_length").alias("foo") > 4.8).select("sepal_length", "sepal_width")
     assert unoptimized.column_names == ["sepal_length", "sepal_width"]
-    assert_plan_eq(optimizer(unoptimized.plan()), optimized.plan())
+    assert_plan_eq(optimizer(unoptimized._get_current_builder()._plan), optimized._get_current_builder()._plan)
 
 
 def test_filter_pushdown_with_column(valid_data: list[dict[str, float]], optimizer) -> None:
@@ -57,7 +57,7 @@ def test_filter_pushdown_with_column(valid_data: list[dict[str, float]], optimiz
     unoptimized = df.with_column("foo", col("sepal_length") + 1).where(col("sepal_length") > 4.8)
     optimized = df.where(col("sepal_length") > 4.8).with_column("foo", col("sepal_length") + 1)
     assert unoptimized.column_names == [*df.column_names, "foo"]
-    assert_plan_eq(optimizer(unoptimized.plan()), optimized.plan())
+    assert_plan_eq(optimizer(unoptimized._get_current_builder()._plan), optimized._get_current_builder()._plan)
 
 
 def test_filter_pushdown_with_column_partial_predicate_pushdown(valid_data: list[dict[str, float]], optimizer) -> None:
@@ -67,7 +67,7 @@ def test_filter_pushdown_with_column_partial_predicate_pushdown(valid_data: list
     )
     optimized = df.where(col("sepal_length") > 4.8).with_column("foo", col("sepal_length") + 1).where(col("foo") > 4.8)
     assert unoptimized.column_names == [*df.column_names, "foo"]
-    assert_plan_eq(optimizer(unoptimized.plan()), optimized.plan())
+    assert_plan_eq(optimizer(unoptimized._get_current_builder()._plan), optimized._get_current_builder()._plan)
 
 
 def test_filter_pushdown_with_column_alias(valid_data: list[dict[str, float]], optimizer) -> None:
@@ -79,7 +79,7 @@ def test_filter_pushdown_with_column_alias(valid_data: list[dict[str, float]], o
         "foo", col("sepal_length").alias("foo") + 1
     )
     assert unoptimized.column_names == [*df.column_names, "foo"]
-    assert_plan_eq(optimizer(unoptimized.plan()), optimized.plan())
+    assert_plan_eq(optimizer(unoptimized._get_current_builder()._plan), optimized._get_current_builder()._plan)
 
 
 def test_filter_merge(valid_data: list[dict[str, float]], optimizer) -> None:
@@ -92,9 +92,9 @@ def test_filter_merge(valid_data: list[dict[str, float]], optimizer) -> None:
         [(col("sepal_width") > 2.4).alias("foo"), (col("sepal_length") > 4.8).alias("foo").alias("copy.foo")]
     )
     optimized = df.where(DUMMY)
-    optimized._plan._predicate = EXPECTED
+    optimized._get_current_builder()._plan._predicate = EXPECTED
 
-    assert_plan_eq(optimizer(unoptimized.plan()), optimized.plan())
+    assert_plan_eq(optimizer(unoptimized._get_current_builder()._plan), optimized._get_current_builder()._plan)
 
 
 def test_filter_pushdown_sort(valid_data: list[dict[str, float]], optimizer) -> None:
@@ -102,7 +102,7 @@ def test_filter_pushdown_sort(valid_data: list[dict[str, float]], optimizer) ->
     unoptimized = df.sort("sepal_length").select("sepal_length", "sepal_width").where(col("sepal_length") > 4.8)
     optimized = df.where(col("sepal_length") > 4.8).sort("sepal_length").select("sepal_length", "sepal_width")
     assert unoptimized.column_names == ["sepal_length", "sepal_width"]
-    assert_plan_eq(optimizer(unoptimized.plan()), optimized.plan())
+    assert_plan_eq(optimizer(unoptimized._get_current_builder()._plan), optimized._get_current_builder()._plan)
 
 
 def test_filter_pushdown_repartition(valid_data: list[dict[str, float]], optimizer) -> None:
@@ -110,7 +110,7 @@ def test_filter_pushdown_repartition(valid_data: list[dict[str, float]], optimiz
     unoptimized = df.repartition(2).select("sepal_length", "sepal_width").where(col("sepal_length") > 4.8)
     optimized = df.where(col("sepal_length") > 4.8).repartition(2).select("sepal_length", "sepal_width")
     assert unoptimized.column_names == ["sepal_length", "sepal_width"]
-    assert_plan_eq(optimizer(unoptimized.plan()), optimized.plan())
+    assert_plan_eq(optimizer(unoptimized._get_current_builder()._plan), optimized._get_current_builder()._plan)
 
 
 def test_filter_join_pushdown(valid_data: list[dict[str, float]], optimizer) -> None:
@@ -122,12 +122,12 @@ def test_filter_join_pushdown(valid_data: list[dict[str, float]], optimizer) ->
     filtered = joined.where(col("sepal_length") > 4.8)
     filtered = filtered.where(col("right.sepal_width") > 4.8)
 
-    optimized = optimizer(filtered.plan())
+    optimized = optimizer(filtered._get_current_builder()._plan)
 
     expected = df1.where(col("sepal_length") > 4.8).join(df2.where(col("sepal_width") > 4.8), on="variety")
     assert isinstance(optimized, Join)
-    assert isinstance(expected.plan(), Join)
-    assert_plan_eq(optimized, expected.plan())
+    assert isinstance(expected._get_current_builder()._plan, Join)
+    assert_plan_eq(optimized, expected._get_current_builder()._plan)
 
 
 def test_filter_join_pushdown_aliases(valid_data: list[dict[str, float]], optimizer) -> None:
@@ -139,15 +139,15 @@ def test_filter_join_pushdown_aliases(valid_data: list[dict[str, float]], optimi
     filtered = joined.where(col("sepal_length").alias("foo") > 4.8)
     filtered = filtered.where(col("right.sepal_width").alias("foo") > 4.8)
 
-    optimized = optimizer(filtered.plan())
+    optimized = optimizer(filtered._get_current_builder()._plan)
 
     expected = df1.where(
         # Filter merging creates a `copy.*` column when merging predicates with the same name
         (col("sepal_length").alias("foo") > 4.8).alias("copy.foo")
     ).join(df2.where(col("sepal_width").alias("foo") > 4.8), on="variety")
     assert isinstance(optimized, Join)
-    assert isinstance(expected.plan(), Join)
-    assert_plan_eq(optimized, expected.plan())
+    assert isinstance(expected._get_current_builder()._plan, Join)
+    assert_plan_eq(optimized, expected._get_current_builder()._plan)
 
 
 def test_filter_join_pushdown_nonvalid(valid_data: list[dict[str, float]], optimizer) -> None:
@@ -158,10 +158,10 @@ def test_filter_join_pushdown_nonvalid(valid_data: list[dict[str, float]], optim
 
     filtered = joined.where(col("right.sepal_width") > col("sepal_length"))
 
-    optimized = optimizer(filtered.plan())
+    optimized = optimizer(filtered._get_current_builder()._plan)
 
     assert isinstance(optimized, Filter)
-    assert_plan_eq(optimized, filtered.plan())
+    assert_plan_eq(optimized, filtered._get_current_builder()._plan)
 
 
 def test_filter_join_pushdown_nonvalid_aliases(valid_data: list[dict[str, float]], optimizer) -> None:
@@ -172,10 +172,10 @@ def test_filter_join_pushdown_nonvalid_aliases(valid_data: list[dict[str, float]
 
     filtered = joined.where(col("right.sepal_width").alias("sepal_width") > col("sepal_length"))
 
-    optimized = optimizer(filtered.plan())
+    optimized = optimizer(filtered._get_current_builder()._plan)
 
     assert isinstance(optimized, Filter)
-    assert_plan_eq(optimized, filtered.plan())
+    assert_plan_eq(optimized, filtered._get_current_builder()._plan)
 
 
 def test_filter_join_partial_predicate_pushdown(valid_data: list[dict[str, float]], optimizer) -> None:
@@ -188,7 +188,7 @@ def test_filter_join_partial_predicate_pushdown(valid_data: list[dict[str, float
     filtered = filtered.where(col("right.sepal_width") > 4.8)
     filtered = filtered.where(((col("sepal_length") > 4.8) | (col("right.sepal_length") > 4.8)).alias("foo"))
 
-    optimized = optimizer(filtered.plan())
+    optimized = optimizer(filtered._get_current_builder()._plan)
 
     expected = (
         df1.where(col("sepal_length") > 4.8)
@@ -196,8 +196,8 @@ def test_filter_join_partial_predicate_pushdown(valid_data: list[dict[str, float
         .where(((col("sepal_length") > 4.8) | (col("right.sepal_length") > 4.8)).alias("foo"))
     )
     assert isinstance(optimized, Filter)
-    assert isinstance(expected.plan(), Filter)
-    assert_plan_eq(optimized, expected.plan())
+    assert isinstance(expected._get_current_builder()._plan, Filter)
+    assert_plan_eq(optimized, expected._get_current_builder()._plan)
 
 
 def test_filter_concat_predicate_pushdown(valid_data, optimizer) -> None:
@@ -205,9 +205,9 @@ def test_filter_concat_predicate_pushdown(valid_data, optimizer) -> None:
     df2 = daft.from_pylist(valid_data)
     concatted = df1.concat(df2)
     filtered = concatted.where(col("sepal_length") > 4.8)
-    optimized = optimizer(filtered.plan())
+    optimized = optimizer(filtered._get_current_builder()._plan)
 
     expected = df1.where(col("sepal_length") > 4.8).concat(df2.where(col("sepal_length") > 4.8))
     assert isinstance(optimized, Concat)
-    assert isinstance(expected.plan(), Concat)
-    assert_plan_eq(optimized, expected.plan())
+    assert isinstance(expected._get_current_builder()._plan, Concat)
+    assert_plan_eq(optimized, expected._get_current_builder()._plan)
